@@ -1,5 +1,5 @@
 /*
-  Copyright 2008-2013 David Robillard <http://drobilla.net>
+  Copyright 2008-2015 David Robillard <http://drobilla.net>
 
   Permission to use, copy, modify, and/or distribute this software for any
   purpose with or without fee is hereby granted, provided that the above
@@ -22,6 +22,12 @@
    This header is non-normative, it is provided for convenience.
 */
 
+/**
+   @defgroup util Utilities
+   @ingroup atom
+   @{
+*/
+
 #ifndef LV2_ATOM_UTIL_H
 #define LV2_ATOM_UTIL_H
 
@@ -41,24 +47,24 @@ extern "C" {
 static inline uint32_t
 lv2_atom_pad_size(uint32_t size)
 {
-	return (size + 7) & (~7);
+	return (size + 7U) & (~7U);
 }
 
-/** Return the total size of @p atom, including the header. */
+/** Return the total size of `atom`, including the header. */
 static inline uint32_t
 lv2_atom_total_size(const LV2_Atom* atom)
 {
-	return sizeof(LV2_Atom) + atom->size;
+	return (uint32_t)sizeof(LV2_Atom) + atom->size;
 }
 
-/** Return true iff @p atom is null. */
+/** Return true iff `atom` is null. */
 static inline bool
 lv2_atom_is_null(const LV2_Atom* atom)
 {
 	return !atom || (atom->type == 0 && atom->size == 0);
 }
 
-/** Return true iff @p a is equal to @p b. */
+/** Return true iff `a` is equal to `b`. */
 static inline bool
 lv2_atom_equals(const LV2_Atom* a, const LV2_Atom* b)
 {
@@ -86,7 +92,7 @@ lv2_atom_sequence_end(const LV2_Atom_Sequence_Body* body, uint32_t size)
 	return (LV2_Atom_Event*)((const uint8_t*)body + lv2_atom_pad_size(size));
 }
 
-/** Return true iff @p i has reached the end of @p body. */
+/** Return true iff `i` has reached the end of `body`. */
 static inline bool
 lv2_atom_sequence_is_end(const LV2_Atom_Sequence_Body* body,
                          uint32_t                      size,
@@ -95,7 +101,7 @@ lv2_atom_sequence_is_end(const LV2_Atom_Sequence_Body* body,
 	return (const uint8_t*)i >= ((const uint8_t*)body + size);
 }
 
-/** Return an iterator to the element following @p i. */
+/** Return an iterator to the element following `i`. */
 static inline LV2_Atom_Event*
 lv2_atom_sequence_next(const LV2_Atom_Event* i)
 {
@@ -127,27 +133,73 @@ lv2_atom_sequence_next(const LV2_Atom_Event* i)
 	     !lv2_atom_sequence_is_end(body, size, (iter)); \
 	     (iter) = lv2_atom_sequence_next(iter))
 
+/**
+   @}
+   @name Sequence Utilities
+   @{
+*/
+
+/**
+   Clear all events from `sequence`.
+
+   This simply resets the size field, the other fields are left untouched.
+*/
+static inline void
+lv2_atom_sequence_clear(LV2_Atom_Sequence* seq)
+{
+	seq->atom.size = sizeof(LV2_Atom_Sequence_Body);
+}
+
+/**
+   Append an event at the end of `sequence`.
+
+   @param seq Sequence to append to.
+   @param capacity Total capacity of the sequence atom
+   (e.g. as set by the host for sequence output ports).
+   @param event Event to write.
+
+   @return A pointer to the newly written event in `seq`,
+   or NULL on failure (insufficient space).
+*/
+static inline LV2_Atom_Event*
+lv2_atom_sequence_append_event(LV2_Atom_Sequence*    seq,
+                               uint32_t              capacity,
+                               const LV2_Atom_Event* event)
+{
+	const uint32_t total_size = (uint32_t)sizeof(*event) + event->body.size;
+	if (capacity - seq->atom.size < total_size) {
+		return NULL;
+	}
+
+	LV2_Atom_Event* e = lv2_atom_sequence_end(&seq->body, seq->atom.size);
+	memcpy(e, event, total_size);
+
+	seq->atom.size += lv2_atom_pad_size(total_size);
+
+	return e;
+}
+
 /**
    @}
    @name Tuple Iterator
    @{
 */
 
-/** Get an iterator pointing to the first element in @p tup. */
+/** Get an iterator pointing to the first element in `tup`. */
 static inline LV2_Atom*
 lv2_atom_tuple_begin(const LV2_Atom_Tuple* tup)
 {
 	return (LV2_Atom*)(LV2_ATOM_BODY(tup));
 }
 
-/** Return true iff @p i has reached the end of @p body. */
+/** Return true iff `i` has reached the end of `body`. */
 static inline bool
 lv2_atom_tuple_is_end(const void* body, uint32_t size, const LV2_Atom* i)
 {
 	return (const uint8_t*)i >= ((const uint8_t*)body + size);
 }
 
-/** Return an iterator to the element following @p i. */
+/** Return an iterator to the element following `i`. */
 static inline LV2_Atom*
 lv2_atom_tuple_next(const LV2_Atom* i)
 {
@@ -162,14 +214,14 @@ lv2_atom_tuple_next(const LV2_Atom* i)
 
    This macro is used similarly to a for loop (which it expands to), e.g.:
    @code
-   LV2_ATOMO_TUPLE_FOREACH(tuple, elem) {
+   LV2_ATOM_TUPLE_FOREACH(tuple, elem) {
        // Do something with elem (an LV2_Atom*) here...
    }
    @endcode
 */
 #define LV2_ATOM_TUPLE_FOREACH(tuple, iter) \
 	for (LV2_Atom* (iter) = lv2_atom_tuple_begin(tuple); \
-	     !lv2_atom_tuple_is_end(LV2_ATOM_BODY(tuple), (tuple)->size, (iter)); \
+	     !lv2_atom_tuple_is_end(LV2_ATOM_BODY(tuple), (tuple)->atom.size, (iter)); \
 	     (iter) = lv2_atom_tuple_next(iter))
 
 /** Like LV2_ATOM_TUPLE_FOREACH but for a headerless tuple body. */
@@ -184,14 +236,14 @@ lv2_atom_tuple_next(const LV2_Atom* i)
    @{
 */
 
-/** Return a pointer to the first property in @p body. */
+/** Return a pointer to the first property in `body`. */
 static inline LV2_Atom_Property_Body*
 lv2_atom_object_begin(const LV2_Atom_Object_Body* body)
 {
 	return (LV2_Atom_Property_Body*)(body + 1);
 }
 
-/** Return true iff @p i has reached the end of @p obj. */
+/** Return true iff `i` has reached the end of `obj`. */
 static inline bool
 lv2_atom_object_is_end(const LV2_Atom_Object_Body*   body,
                        uint32_t                      size,
@@ -200,15 +252,15 @@ lv2_atom_object_is_end(const LV2_Atom_Object_Body*   body,
 	return (const uint8_t*)i >= ((const uint8_t*)body + size);
 }
 
-/** Return an iterator to the property following @p i. */
+/** Return an iterator to the property following `i`. */
 static inline LV2_Atom_Property_Body*
 lv2_atom_object_next(const LV2_Atom_Property_Body* i)
 {
 	const LV2_Atom* const value = (const LV2_Atom*)(
 		(const uint8_t*)i + 2 * sizeof(uint32_t));
 	return (LV2_Atom_Property_Body*)(
-		(const uint8_t*)i + lv2_atom_pad_size(sizeof(LV2_Atom_Property_Body)
-		                                + value->size));
+		(const uint8_t*)i + lv2_atom_pad_size(
+			(uint32_t)sizeof(LV2_Atom_Property_Body) + value->size));
 }
 
 /**
@@ -251,10 +303,10 @@ static const LV2_Atom_Object_Query LV2_ATOM_OBJECT_QUERY_END = { 0, NULL };
 /**
    Get an object's values for various keys.
 
-   The value pointer of each item in @p query will be set to the location of
-   the corresponding value in @p object.  Every value pointer in @p query MUST
-   be initialised to NULL.  This function reads @p object in a single linear
-   sweep.  By allocating @p query on the stack, objects can be "queried"
+   The value pointer of each item in `query` will be set to the location of
+   the corresponding value in `object`.  Every value pointer in `query` MUST
+   be initialised to NULL.  This function reads `object` in a single linear
+   sweep.  By allocating `query` on the stack, objects can be "queried"
    quickly without allocating any memory.  This function is realtime safe.
 
    This function can only do "flat" queries, it is not smart enough to match
@@ -391,6 +443,7 @@ lv2_atom_object_get(const LV2_Atom_Object* object, ...)
 
 /**
    @}
+   @}
 */
 
 #ifdef __cplusplus
