@@ -1,6 +1,6 @@
 /*
   LV2 Sampler Example Plugin UI
-  Copyright 2011-2012 David Robillard <d@drobilla.net>
+  Copyright 2011-2016 David Robillard <d@drobilla.net>
 
   Permission to use, copy, modify, and/or distribute this software for any
   purpose with or without fee is hereby granted, provided that the above
@@ -22,9 +22,11 @@
 #include "lv2/lv2plug.in/ns/ext/atom/atom.h"
 #include "lv2/lv2plug.in/ns/ext/atom/forge.h"
 #include "lv2/lv2plug.in/ns/ext/atom/util.h"
+#include "lv2/lv2plug.in/ns/ext/log/logger.h"
 #include "lv2/lv2plug.in/ns/ext/patch/patch.h"
 #include "lv2/lv2plug.in/ns/ext/urid/urid.h"
 #include "lv2/lv2plug.in/ns/extensions/ui/ui.h"
+#include "lv2/lv2plug.in/ns/lv2core/lv2_util.h"
 
 #include "./uris.h"
 
@@ -32,9 +34,9 @@
 
 typedef struct {
 	LV2_Atom_Forge forge;
-
-	LV2_URID_Map* map;
-	SamplerURIs   uris;
+	LV2_URID_Map*  map;
+	LV2_Log_Logger logger;
+	SamplerURIs    uris;
 
 	LV2UI_Write_Function write;
 	LV2UI_Controller     controller;
@@ -76,8 +78,8 @@ on_load_clicked(GtkWidget* widget,
 	uint8_t obj_buf[OBJ_BUF_SIZE];
 	lv2_atom_forge_set_buffer(&ui->forge, obj_buf, OBJ_BUF_SIZE);
 
-	LV2_Atom* msg = write_set_file(&ui->forge, &ui->uris,
-	                               filename, strlen(filename));
+	LV2_Atom* msg = (LV2_Atom*)write_set_file(&ui->forge, &ui->uris,
+	                                          filename, strlen(filename));
 
 	ui->write(ui->controller, 0, lv2_atom_total_size(msg),
 	          ui->uris.atom_eventTransfer,
@@ -95,33 +97,33 @@ instantiate(const LV2UI_Descriptor*   descriptor,
             LV2UI_Widget*             widget,
             const LV2_Feature* const* features)
 {
-	SamplerUI* ui = (SamplerUI*)malloc(sizeof(SamplerUI));
-	ui->map        = NULL;
-	ui->write      = write_function;
-	ui->controller = controller;
-	ui->box        = NULL;
-	ui->button     = NULL;
-	ui->label      = NULL;
-	ui->window     = NULL;
-
-	*widget = NULL;
-
-	for (int i = 0; features[i]; ++i) {
-		if (!strcmp(features[i]->URI, LV2_URID_URI "#map")) {
-			ui->map = (LV2_URID_Map*)features[i]->data;
-		}
+	SamplerUI* ui = (SamplerUI*)calloc(1, sizeof(SamplerUI));
+	if (!ui) {
+		return NULL;
 	}
 
-	if (!ui->map) {
-		fprintf(stderr, "sampler_ui: Host does not support urid:Map\n");
+	ui->write      = write_function;
+	ui->controller = controller;
+	*widget        = NULL;
+
+	// Get host features
+	const char* missing = lv2_features_query(
+		features,
+		LV2_LOG__log,  &ui->logger.log, false,
+		LV2_URID__map, &ui->map,        true,
+		NULL);
+	lv2_log_logger_set_map(&ui->logger, ui->map);
+	if (missing) {
+		lv2_log_error(&ui->logger, "Missing feature <%s>\n", missing);
 		free(ui);
 		return NULL;
 	}
 
+	// Map URIs and initialise forge
 	map_sampler_uris(ui->map, &ui->uris);
-
 	lv2_atom_forge_init(&ui->forge, ui->map);
 
+	// Construct Gtk UI
 	ui->box = gtk_vbox_new(FALSE, 4);
 	ui->label = gtk_label_new("?");
 	ui->button = gtk_button_new_with_label("Load Sample");
@@ -168,20 +170,18 @@ port_event(LV2UI_Handle handle,
 	if (format == ui->uris.atom_eventTransfer) {
 		const LV2_Atom* atom = (const LV2_Atom*)buffer;
 		if (lv2_atom_forge_is_object_type(&ui->forge, atom->type)) {
-			const LV2_Atom_Object* obj      = (const LV2_Atom_Object*)atom;
-			const LV2_Atom*        file_uri = read_set_file(&ui->uris, obj);
-			if (!file_uri) {
-				fprintf(stderr, "Unknown message sent to UI.\n");
-				return;
+			const LV2_Atom_Object* obj = (const LV2_Atom_Object*)atom;
+			const char*            uri = read_set_file(&ui->uris, obj);
+			if (uri) {
+				gtk_label_set_text(GTK_LABEL(ui->label), uri);
+			} else {
+				lv2_log_warning(&ui->logger, "Malformed message\n");
 			}
-
-			const char* uri = (const char*)LV2_ATOM_BODY_CONST(file_uri);
-			gtk_label_set_text(GTK_LABEL(ui->label), uri);
 		} else {
-			fprintf(stderr, "Unknown message type.\n");
+			lv2_log_error(&ui->logger, "Unknown message type\n");
 		}
 	} else {
-		fprintf(stderr, "Unknown format.\n");
+		lv2_log_warning(&ui->logger, "Unknown port event format\n");
 	}
 }
 
